{******************************************************************************}
{                                                                              }
{ Project JEDI                                                                 }
{ Sample program donated to Michael DeVault  DeVaSys.                          }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is Main.pas.                                               }
{                                                                              }
{ The Initial Developer of the Original Code is Robert Marquardt.              }
{ Copyright (C) 2001 Robert Marquardt                                          }
{                                                                              }
{******************************************************************************}

unit Main;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls,
  Forms, StdCtrls, ExtCtrls, Buttons, Menus, ComCtrls, jpeg,
  DBT, usbi2cio;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileExitItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDevice1: TMenuItem;
    CloseDevice1: TMenuItem;
    PageControl1: TPageControl;
    DigitalIO: TTabSheet;
    I2CBus: TTabSheet;
    PortABox: TGroupBox;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Image5: TImage;
    Image6: TImage;
    Image7: TImage;
    Image8: TImage;
    Image9: TImage;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Bitlabel7: TLabel;
    Bitlabel6: TLabel;
    Bitlabel5: TLabel;
    Bitlabel4: TLabel;
    Bitlabel3: TLabel;
    Bitlabel2: TLabel;
    Bitlabel1: TLabel;
    Bitlabel0: TLabel;
    AHex: TLabel;
    ADec: TLabel;
    PortBBox: TGroupBox;
    Image10: TImage;
    Image11: TImage;
    Image12: TImage;
    Image13: TImage;
    Image14: TImage;
    Image15: TImage;
    Image16: TImage;
    Image17: TImage;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Bitlabel11: TLabel;
    Bitlabel10: TLabel;
    Bitlabel9: TLabel;
    Bitlabel8: TLabel;
    Bitlabel12: TLabel;
    Bitlabel13: TLabel;
    Bitlabel14: TLabel;
    Bitlabel15: TLabel;
    BDec: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    BHex: TLabel;
    PortCBox: TGroupBox;
    Image18: TImage;
    Image19: TImage;
    Image20: TImage;
    Image21: TImage;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Bitlabel20: TLabel;
    Bitlabel21: TLabel;
    Bitlabel22: TLabel;
    Bitlabel23: TLabel;
    CDec: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    CHex: TLabel;
    DeViceBox: TGroupBox;
    Image1: TImage;
    PinPolling: TSpeedButton;
    PollTimer: TTimer;
    I2CRead: TButton;
    I2CWrite: TButton;
    Device: TStaticText;
    Events: TStaticText;
    Label1: TLabel;
    DevAddress: TEdit;
    Label31: TLabel;
    AddressingMode: TComboBox;
    Label4: TLabel;
    MemAddress: TEdit;
    Label34: TLabel;
    DataEntry: TEdit;
    Image25: TImage;
    Image24: TImage;
    Image23: TImage;
    Image22: TImage;
    DataCount: TEdit;
    Label35: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RefreshDisplay;
    procedure FileExit(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure OpenDevice1Click(Sender: TObject);
    procedure CloseDevice1Click(Sender: TObject);
    procedure ArrowClick(Sender: TObject);
    procedure BitClick(Sender: TObject);
    procedure LabelClick(Sender: TObject);
    procedure PinPollingClick(Sender: TObject);
    procedure AddressingModeChange(Sender: TObject);
    procedure MemAddressEnter(Sender: TObject);
    procedure PollTimerTimer(Sender: TObject);
    procedure DataEntryEnter(Sender: TObject);
    procedure I2CReadClick(Sender: TObject);
    procedure I2CWriteClick(Sender: TObject);
    procedure FillInI2CData;
  private
    // some arrays to handle controls
    Arrows:         array [0..31] of TImage;
    BitLabels:      array [0..31] of TLabel;
    DataEntryEdits: array [0..63] of TEdit;
    // get any device plugs or unplugs
    procedure WMDevChange(var Msg: TWMDeviceChange); message WM_DEVICECHANGE;
  public
    // global variables to access the device
    NumDevices:   Byte;        // number of devices currently plugged
    DriverName:   string;      // symbolic name of the device driver
    DeviceInfo:   TDevInfo;    // info for the currently opened device
    DeviceHandle: THandle;     // file handle to the device
    DataBits:     DWORD;       // currently shown bits of the device
    IODirections: DWORD;       // directions of the digital IO lines
    PollCounter:  Integer;     // a counter for the reads of the digital IO lines
    I2CData:      TI2CTrans;   // data block for read and write of I2C bus
  end;

var
  MainForm: TMainForm;

implementation

uses Splash, Select;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
  J: Integer;
begin
  // init globals to "no device"
  NumDevices   := 0;
  DriverName   := '';
  FillChar(DeviceInfo, SizeOf(DeviceInfo), 0);
  DeviceHandle := INVALID_HANDLE_VALUE;
  DataBits     := 0;
  IODirections := 0;
  PollCounter  := 0;
  FillChar(I2CData, SizeOf(I2CData), 0);

  // initialize all controls
  AddressingMode.ItemIndex := 0;

  // put all arrows in an array
  // to be able to index them
  Arrows[ 0] := Image2;
  Arrows[ 1] := Image3;
  Arrows[ 2] := Image4;
  Arrows[ 3] := Image5;
  Arrows[ 4] := Image6;
  Arrows[ 5] := Image7;
  Arrows[ 6] := Image8;
  Arrows[ 7] := Image9;

  Arrows[ 8] := Image10;
  Arrows[ 9] := Image11;
  Arrows[10] := Image12;
  Arrows[11] := Image13;
  Arrows[12] := Image14;
  Arrows[13] := Image15;
  Arrows[14] := Image16;
  Arrows[15] := Image17;

  Arrows[16] := Image18;
  Arrows[17] := Image19;
  Arrows[18] := Image20;
  Arrows[19] := Image21;
  Arrows[20] := nil;
  Arrows[21] := nil;
  Arrows[22] := nil;
  Arrows[23] := nil;

  // initialize arrows
  CloseDevice1Click(Self);

  // put all bit labels in an array
  // to be able to index them in LabelClick
  BitLabels[ 0] := Bitlabel0;
  BitLabels[ 1] := Bitlabel1;
  BitLabels[ 2] := Bitlabel2;
  BitLabels[ 3] := Bitlabel3;
  BitLabels[ 4] := Bitlabel4;
  BitLabels[ 5] := Bitlabel5;
  BitLabels[ 6] := Bitlabel6;
  BitLabels[ 7] := Bitlabel7;

  BitLabels[ 8] := Bitlabel8;
  BitLabels[ 9] := Bitlabel9;
  BitLabels[10] := Bitlabel10;
  BitLabels[11] := Bitlabel11;
  BitLabels[12] := Bitlabel12;
  BitLabels[13] := Bitlabel13;
  BitLabels[14] := Bitlabel14;
  BitLabels[15] := Bitlabel15;

  BitLabels[16] := Bitlabel20;
  BitLabels[17] := Bitlabel21;
  BitLabels[18] := Bitlabel22;
  BitLabels[19] := Bitlabel23;
  BitLabels[20] := nil;
  BitLabels[21] := nil;
  BitLabels[22] := nil;
  BitLabels[23] := nil;

  RefreshDisplay;

  // load the empty arrows from resources
  Image22.Picture.Bitmap.LoadFromResourceID(HInstance, 103);
  Image23.Picture.Bitmap.LoadFromResourceID(HInstance, 103);
  Image24.Picture.Bitmap.LoadFromResourceID(HInstance, 103);
  Image25.Picture.Bitmap.LoadFromResourceID(HInstance, 103);

  // create the 64 data entry edit boxes by cloning DataEntry
  DataEntryEdits[0] := DataEntry;
  // create the first line of controls (only 15 new ones)
  for J := 1 to 15 do
  begin
    DataEntryEdits[J] := TEdit.Create(DataEntry.Owner);
    with DataEntryEdits[J] do
    begin
      // take the settings from the template DataEntry
      Parent    := DataEntry.Parent;
      Top       := DataEntry.Top;
      Left      := DataEntry.Left + J*22;
      Width     := DataEntry.Width;
      OnEnter   := DataEntry.OnEnter;
      MaxLength := 2;
      Visible   := True;
    end;
  end;
  // now create the 3 other lines (16 controls each)
  for I := 1 to 3 do
    for J := 0 to 15 do
    begin
      DataEntryEdits[I*16+J] := TEdit.Create(DataEntry.Owner);
      with DataEntryEdits[I*16+J] do
      begin
        Parent    := DataEntry.Parent;
        Top       := DataEntry.Top  + I*24;
        Left      := DataEntry.Left + J*22;
        Width     := DataEntry.Width;
        OnEnter   := DataEntry.OnEnter;
        MaxLength := 2;
        Visible   := True;
      end;
    end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  // close the device for cleanup
  CloseDevice1Click(Self);
end;

procedure TMainForm.WMDevChange(var Msg: TWMDeviceChange);
begin
  // a device has been plugged or unplugged
  if Msg.Event = DBT_DEVNODES_CHANGED then
    // if a device is opened
    if DeviceHandle <> INVALID_HANDLE_VALUE then
      // check if the device is still present
      if not DAPI_DetectDevice(DeviceHandle) then
        // if not close it for cleanup
        CloseDevice1Click(Self);
end;

procedure TMainForm.RefreshDisplay;
var
  I: Integer;
begin
  // show the IO bits of the device
  if DeviceHandle <> INVALID_HANDLE_VALUE then
    // read fresh bits if there is a opened device
    DAPI_ReadIOPorts(DeviceHandle, DataBits);
  // show the bits on the arrows
  for I := 0 to 31 do
    if Assigned(BitLabels[I]) then
      if (DataBits and (1 shl I)) <> 0 then
        BitLabels[I].Caption := '1'
      else
        BitLabels[I].Caption := '0';
  // show the poll counter (nicely positioned a bit to the left)
  Events.Caption := Format('%u  ', [PollCounter]);
  // show the current bit combination as numbers
  AHex.Caption := Format('$%.2x', [DataBits          and $ff]);
  ADec.Caption := Format('%u',    [DataBits          and $ff]);
  BHex.Caption := Format('$%.2x', [(DataBits shr  8) and $ff]);
  BDec.Caption := Format('%u',    [(DataBits shr  8) and $ff]);
  CHex.Caption := Format('$%.2x', [(DataBits shr 16) and $ff]);
  CDec.Caption := Format('%u',    [(DataBits shr 16) and $ff]);
end;

procedure TMainForm.FileExit(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.HelpAboutItemClick(Sender: TObject);
var
  Splash: TSplashScreen;
begin
  // show the Splash Screen as About window without
  // activating the timeout timer
  Splash := TSplashScreen.Create(Self);
  Splash.ShowModal;
  Splash.Free;
end;

procedure TMainForm.OpenDevice1Click(Sender: TObject);
begin
  // show the "Select device" dialog
  with TSelectDevice.Create(Self) do
  begin
    if ShowModal = mrOK then
    begin
      // close any previously opened device
      CloseDevice1Click(Self);
      // open the device and update the global variables
      Self.NumDevices := NumDevices;
      DriverName      := SymbolicName.Text;
      DeviceInfo      := DeviceInfos[AttachedDevices.ItemIndex];
      DeviceHandle    := DAPI_OpenDeviceInstance(PChar(DriverName), DeviceInfo.byInstance);
      if DeviceHandle <> INVALID_HANDLE_VALUE then
      begin
        // show some info
        MainForm.Caption := 'EzIo Delphi - ' +
          AttachedDevices.Items[AttachedDevices.ItemIndex];
        Device.Caption := AttachedDevices.Items[AttachedDevices.ItemIndex];
        Device.Color   := clLime;
        // change the state of various controls
        CloseDevice1.Enabled := True;
        PinPolling.Enabled   := True;
        I2CRead.Enabled      := True;
        I2CWrite.Enabled     := True;
        RefreshDisplay;
      end
      else
        CloseDevice1Click(Self);
    end;
    Free;
  end;
end;

procedure TMainForm.CloseDevice1Click(Sender: TObject);
var
  I: Integer;
begin
  // close any open device handle
  if DeviceHandle <> INVALID_HANDLE_VALUE then
    DAPI_CloseDeviceInstance(DeviceHandle);
  DeviceHandle := INVALID_HANDLE_VALUE;

  DataBits     := 0;
  IODirections := $00000000;   // set IO directions as output
  // kick all arrows to turn their direction bit and show their image
  for I := 0 to 31 do
    if Assigned(Arrows[I]) then
      ArrowClick(Arrows[I]);
  // show some info (closed state)
  MainForm.Caption := 'EzIo Delphi - no device';
  Device.Caption := 'Device';
  Device.Color := clRed;
  // change the state of various controls
  CloseDevice1.Enabled := False;
  PinPolling.Enabled   := False;
  I2CRead.Enabled      := False;
  I2CWrite.Enabled     := False;
end;

//-- Digital IO ---------------------------------------------

procedure TMainForm.ArrowClick(Sender: TObject);
var
  Img: TImage;
  Num: DWORD;
begin
  // bit number is stored in image
  Img := TImage(Sender);
  Num := 1 shl Img.Tag;
  // toggle input/output and show corresponding arrow
  if (IODirections and Num) <> 0 then
  begin
    IODirections := IODirections and not Num;
    Img.Picture.Bitmap.LoadFromResourceID(HInstance, 102);
  end
  else
  begin
    IODirections := IODirections or Num;
    Img.Picture.Bitmap.LoadFromResourceID(HInstance, 101);
  end;
  if DeviceHandle <> INVALID_HANDLE_VALUE then
  begin
    // send the direction change to the device
    DAPI_ConfigIOPorts(DeviceHandle, IODirections);
    // output the bits
    DAPI_WriteIOPorts(DeviceHandle, DataBits, (not IODirections) and $00FFFFFF);
  end;
  // updates the window to show the new state (also reads the bits)
  RefreshDisplay;
end;

procedure TMainForm.BitClick(Sender: TObject);
var
  Bit: TLabel;
  Num: DWORD;
begin
  // bit number is stored in label
  Bit := TLabel(Sender);
  Num := 1 shl Bit.Tag;
  // if input line then do nothing
  if (IODirections and Num) <> 0 then
    Exit;
  // toggle data bit and show corresponding label
  if (DataBits and Num) <> 0 then
    DataBits := DataBits and not Num
  else
    DataBits := DataBits or Num;
  if DeviceHandle <> INVALID_HANDLE_VALUE then
    // output the changed bits
    DAPI_WriteIOPorts(DeviceHandle, DataBits, (not IODirections) and $00FFFFFF);
  // updates the window to show the new state (also reads the bits)
  RefreshDisplay;
end;

procedure TMainForm.LabelClick(Sender: TObject);
begin
  // let the corresponding bit label do the work
  BitClick(Bitlabels[TLabel(Sender).Tag]);
end;

procedure TMainForm.PinPollingClick(Sender: TObject);
begin
  // if the button has gone down then reset the counter
  if PinPolling.Down then
    PollCounter := 0;
  // start/stop the read timer
  PollTimer.Enabled := PinPolling.Down;
end;

procedure TMainForm.PollTimerTimer(Sender: TObject);
begin
  // increment the poll counter
  Inc(PollCounter);
  // updates the window to show the new state (also reads the bits)
  RefreshDisplay;
end;

//-- I2C ----------------------------------------------------

procedure TMainForm.AddressingModeChange(Sender: TObject);
begin
  // when the addressing mode combobox changes then
  // kick the mem address to reformat
  MemAddressEnter(Sender);
end;

procedure TMainForm.MemAddressEnter(Sender: TObject);
var
  N: Integer;
begin
  // disable if "no addess" is selected
  MemAddress.Enabled := AddressingMode.ItemIndex <> 0;
  // reformat to 2 or 4 digit address
  N := StrToIntDef('$' + MemAddress.Text, 0);
  case AddressingMode.ItemIndex of
    0: MemAddress.Text := '00';
    1: MemAddress.Text := IntToHex(N and $FF, 2);
    2: MemAddress.Text := IntToHex(N and $FFFF, 4);
  end;
end;

procedure TMainForm.DataEntryEnter(Sender: TObject);
var
  N: Integer;
begin
  // reformat to 2 digits
  // changes to "00" if empty
  N := StrToIntDef('$' + TEdit(Sender).Text, 0);
  TEdit(Sender).Text := IntToHex(N, 2);
end;

procedure TMainForm.FillInI2CData;
begin
  // transfers the values from the controls to the data structure
  FillChar(I2CData, SizeOf(I2CData), #0);
  DataEntryEnter(DevAddress);
  MemAddressEnter(MemAddress);
  DataEntryEnter(DataCount);
  I2CData.bySlvDevAddr := StrToIntDef('$' + DevAddress.Text, 0);
  case AddressingMode.ItemIndex of
    0: I2CData.byTransType := I2C_TRANS_NOADR;
    1: I2CData.byTransType := I2C_TRANS_8ADR;
    2: I2CData.byTransType := I2C_TRANS_16ADR;
  end;
  I2CData.wMemoryAddr := StrToIntDef('$' + MemAddress.Text, 0);
  I2CData.wCount      := StrToIntDef('$' + DataCount.Text, 0);
  // limit all read/write to 64
  if I2CData.wCount > 64 then
  begin
    I2CData.wCount := 64;
    DataCount.Text := IntToHex(I2CData.wCount, 2);
  end;
end;

procedure TMainForm.I2CReadClick(Sender: TObject);
var
  I: Integer;
begin
  // pull the data from the controls
  FillInI2CData;
  // clear all data edit boxes
  for I := 0 to 63 do
    DataEntryEdits[I].Text := '';
  if DeviceHandle <> INVALID_HANDLE_VALUE then
    // read I2C
    if DAPI_ReadI2C(DeviceHandle, I2CData) > 0 then
      // show the read data values
      for I := 0 to I2CData.wCount-1 do
        DataEntryEdits[I].Text := IntToHex(I2CData.Data[I], 2);
end;

procedure TMainForm.I2CWriteClick(Sender: TObject);
var
  I: Integer;
begin
  // pull the data from the controls
  FillInI2CData;
  // refresh the data edits with data
  if I2CData.wCount > 0 then
    for I := 0 to I2CData.wCount-1 do
      DataEntryEnter(DataEntryEdits[I]);
  // clear the data edits without data
  for I := I2CData.wCount to 63 do
    DataEntryEdits[I].Text := '';
  // pull the data
  for I := 0 to I2CData.wCount-1 do
    I2CData.Data[I] := StrToIntDef('$' + DataEntryEdits[I].Text, 0);
  // write the data
  if DeviceHandle <> INVALID_HANDLE_VALUE then
    DAPI_WriteI2C(DeviceHandle, I2CData);
end;

end.
