{******************************************************************************}
{                                                                              }
{ Project JEDI                                                                 }
{ Sample program donated to Michael DeVault  DeVaSys.                          }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is Select.pas.                                             }
{                                                                              }
{ The Initial Developer of the Original Code is Robert Marquardt.              }
{ Copyright (C) 2001 Robert Marquardt                                          }
{                                                                              }
{******************************************************************************}

unit Select;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, StdCtrls,
  DBT, usbi2cio;

type
  TSelectDevice = class(TForm)
    Label1: TLabel;
    SymbolicName: TEdit;
    Attached: TLabel;
    AttachedDevices: TComboBox;
    OKButton: TButton;
    CancelButton: TButton;
    procedure FormCreate(Sender: TObject);
  private
    procedure WMDevChange(var Msg: TWMDeviceChange); message WM_DEVICECHANGE;
  public
    NumDevices: Byte;
    DeviceInfos: array [0..128] of TDevInfo;
    procedure ListDevices;
  end;

implementation

{$R *.DFM}

procedure TSelectDevice.FormCreate(Sender: TObject);
begin
  ListDevices;
end;

procedure TSelectDevice.ListDevices;
var
  I: Byte;
begin
  // refill the list of plugged devices
  AttachedDevices.Clear;
  NumDevices := DAPI_GetDeviceInfo(PChar(SymbolicName.Text), @DeviceInfos[0]);
  // only allow OK if a device is plugged
  OkButton.Enabled := (NumDevices > 0);
  if NumDevices = 0 then
    AttachedDevices.Items.Add('no devices connected')
  else
    for I := 0 to NumDevices-1 do
      AttachedDevices.Items.Add(SymbolicName.Text +
        Format(' %u', [DeviceInfos[I].byInstance]) +
        ' - ' + PChar(@DeviceInfos[I].SerialId[0]));
  AttachedDevices.ItemIndex := 0;
  Attached.Caption := Format('Attached devices: %u', [NumDevices]);
end;

procedure TSelectDevice.WMDevChange(var Msg: TWMDeviceChange);
begin
  // refill list if any device is plugged or unplugged
  if Msg.Event = DBT_DEVNODES_CHANGED then
    ListDevices;
end;

end.
