unit usbi2cio;

interface

uses
  Windows;

const
  DAPI_MAX_DEVICES = 127;

//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------
const
  // supported transaction types
  I2C_TRANS_NOADR = 0;          // read or write with no address cycle
  I2C_TRANS_8ADR  = 1;          // read or write with 8 bit address cycle
  I2C_TRANS_16ADR = 2;          // read or write with 16 bit address cycle
type
  I2C_TRANS_TYPE = Cardinal;
  TI2CTransType  = I2C_TRANS_TYPE;

//-----------------------------------------------------------------------------
// Structure Definitions
//-----------------------------------------------------------------------------
type
  PDevInfo = ^TDevInfo;
  DEVINFO = packed record             // structure for device information
    byInstance: Byte;
    SerialId: array [0..8] of Byte;
  end;
  TDevInfo = DEVINFO;
  LPDEVINFO = PDevInfo;

  PI2CTrans = ^TI2CTrans;
  I2C_TRANS = packed record
    byTransType: Byte;
    bySlvDevAddr: Byte;
    wMemoryAddr: WORD;
    wCount: WORD;
    Data: array [0..255] of Byte;
  end;
  TI2CTrans = I2C_TRANS;
  LPI2C_TRANS = PI2CTrans;

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Macros
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// API Function Prototypes (exported)
//-----------------------------------------------------------------------------

function  DAPI_GetDllVersion: WORD; stdcall;
function  DAPI_OpenDeviceInstance(lpsDevName: PChar; byDevInstance: Byte): THandle; stdcall;
function  DAPI_CloseDeviceInstance(hDevInstance: THandle): BOOL; stdcall;
function  DAPI_DetectDevice(hDevInstance: THandle): BOOL; stdcall;
function  DAPI_GetDeviceCount(lpsDevName: PChar): Byte; stdcall;
function  DAPI_GetDeviceInfo(lpsDevName: PChar; DevInfo: PDevInfo): Byte; stdcall;
function  DAPI_OpenDeviceBySerialId(lpsDevName: PChar; lpsDevSerialId: PChar): THandle; stdcall;
function  DAPI_GetSerialId(hDevInstance: THandle; lpsDevSerialId: PChar): BOOL; stdcall;
function  DAPI_ConfigIoPorts(hDevInstance: THandle; ulIoPortConfig: ULONG): BOOL; stdcall;
function  DAPI_GetIoConfig(hDevInstance: THandle; var ulIoPortData: ULONG): BOOL; stdcall;
function  DAPI_ReadIoPorts(hDevInstance: THandle; var ulIoPortData: ULONG): BOOL; stdcall;
function  DAPI_WriteIoPorts(hDevInstance: THandle; ulIoPortData: ULONG; ulIoPortMask: ULONG): BOOL; stdcall;
function  DAPI_ReadI2c(hDevInstance: THandle; var TransI2C: TI2CTrans): Longint; stdcall;
function  DAPI_WriteI2c(hDevInstance: THandle; var TransI2C: TI2CTrans): Longint; stdcall;
procedure DAPI_EnablePolling; stdcall;
procedure DAPI_DisablePolling; stdcall;
procedure DAPI_GetPolledInfo; stdcall;
function  DAPI_ReadDebugBuffer(lpsDebugString: PChar; hDevInstance: THandle; ulMaxBytes: ULONG): Longint; stdcall;

implementation

const
  I2CModuleName = 'USBI2CIO.DLL';

function  DAPI_GetDllVersion;        external I2CModuleName name 'DAPI_GetDllVersion';
function  DAPI_OpenDeviceInstance;   external I2CModuleName name 'DAPI_OpenDeviceInstance';
function  DAPI_CloseDeviceInstance;  external I2CModuleName name 'DAPI_CloseDeviceInstance';
function  DAPI_DetectDevice;         external I2CModuleName name 'DAPI_DetectDevice';
function  DAPI_GetDeviceCount;       external I2CModuleName name 'DAPI_GetDeviceCount';
function  DAPI_GetDeviceInfo;        external I2CModuleName name 'DAPI_GetDeviceInfo';
function  DAPI_OpenDeviceBySerialId; external I2CModuleName name 'DAPI_OpenDeviceBySerialId';
function  DAPI_GetSerialId;          external I2CModuleName name 'DAPI_GetSerialId';
function  DAPI_ConfigIoPorts;        external I2CModuleName name 'DAPI_ConfigIoPorts';
function  DAPI_GetIoConfig;          external I2CModuleName name 'DAPI_GetIoConfig';
function  DAPI_ReadIoPorts;          external I2CModuleName name 'DAPI_ReadIoPorts';
function  DAPI_WriteIoPorts;         external I2CModuleName name 'DAPI_WriteIoPorts';
function  DAPI_ReadI2c;              external I2CModuleName name 'DAPI_ReadI2c';
function  DAPI_WriteI2c;             external I2CModuleName name 'DAPI_WriteI2c';
procedure DAPI_EnablePolling;        external I2CModuleName name 'DAPI_EnablePolling';
procedure DAPI_DisablePolling;       external I2CModuleName name 'DAPI_DisablePolling';
procedure DAPI_GetPolledInfo;        external I2CModuleName name 'DAPI_GetPolledInfo';
function  DAPI_ReadDebugBuffer;      external I2CModuleName name 'DAPI_ReadDebugBuffer';

end.
